from __future__ import annotations

import pytest
from django.core.exceptions import ImproperlyConfigured
from pydantic import BaseModel
from rest_framework.serializers import ListSerializer

from restdoctor.rest_framework.serializers import PydanticSerializer


def test_pydantic_serializer_without_model_error():
    class InvalidSerializerNoModel(PydanticSerializer):
        pass

    with pytest.raises(AttributeError) as exc:
        InvalidSerializerNoModel()

    assert exc.value.args[0] == (
        'Class attribute "pydantic_model" is mandatory for this serializer'
    )


def test_pydantic_serializer_invalid_model_error():
    class InvalidSerializerInvalidModel(PydanticSerializer):
        pydantic_model = object

    with pytest.raises(AttributeError) as exc:
        InvalidSerializerInvalidModel()

    assert exc.value.args[0] == (
        'Class attribute "pydantic_model" must be an instance of pydantic.BaseModel'
    )


def test_pydantic_model_serializer_successful_initialization(pydantic_model_test_serializer):
    pydantic_model_test_serializer()

    assert True  # exception was not thrown


def test_pydantic_model_serializer_to_internal_value(
    pydantic_model_test_serializer,
    pydantic_test_model,
    pydantic_test_model_data,
    serialized_pydantic_test_model_data,
):
    serializer = pydantic_model_test_serializer()

    internal_data = serializer.to_internal_value(pydantic_test_model_data)

    assert isinstance(internal_data, pydantic_test_model)
    assert internal_data.dict() == serialized_pydantic_test_model_data


@pytest.mark.parametrize(
    'argtype', ['serializer', 'model', 'dict'], ids=['with_serializer', 'with_model', 'with_dict']
)
def test_pydantic_model_serializer_to_representation_success(
    argtype,
    pydantic_model_test_serializer,
    pydantic_test_model,
    pydantic_test_model_data,
    serialized_pydantic_test_model_data,
):
    serializer = pydantic_model_test_serializer()
    argtype_mapping = {
        'serializer': pydantic_model_test_serializer(data=pydantic_test_model_data),
        'model': pydantic_test_model(**pydantic_test_model_data),
        'dict': pydantic_test_model_data,
    }

    representation = serializer.to_representation(argtype_mapping[argtype])

    assert representation == serialized_pydantic_test_model_data


def test_pydantic_model_serializer_to_representation_type_error(pydantic_model_test_serializer):
    serializer = pydantic_model_test_serializer()

    with pytest.raises(TypeError) as exc:
        serializer.to_representation('foo')

    assert exc.value.args[0] == 'Unknown type of instance for representation'


def test_pydantic_model_serializer_is_valid_success(
    pydantic_model_test_serializer, pydantic_test_model_data, serialized_pydantic_test_model_data
):
    serializer = pydantic_model_test_serializer(data=pydantic_test_model_data)

    valid = serializer.is_valid()

    assert valid is True
    assert serializer.data == serialized_pydantic_test_model_data
    assert serializer.errors == {}


def test_pydantic_model_serializer_is_valid_errors(pydantic_model_test_serializer):
    corrupted_data = {'field_a': 1, 'field_b': 'wrong type', 'created_at': 'wrong type'}
    serializer = pydantic_model_test_serializer(data=corrupted_data)

    valid = serializer.is_valid()

    assert valid is False
    assert serializer.errors == {
        'created_at': 'invalid datetime format',
        'field_a': 'str type expected',
        'field_b': 'value is not a valid integer',
    }


def test_pydantic_model_serializer_list(
    pydantic_model_test_serializer, pydantic_test_model_data, serialized_pydantic_test_model_data
):
    list_data = [pydantic_test_model_data, pydantic_test_model_data.copy()]
    expected_data = [
        serialized_pydantic_test_model_data,
        serialized_pydantic_test_model_data.copy(),
    ]
    serializer = pydantic_model_test_serializer(data=list_data, many=True)

    valid = serializer.is_valid()

    assert valid is True
    assert isinstance(serializer, ListSerializer)
    assert isinstance(serializer.child, pydantic_model_test_serializer)
    assert serializer.data == expected_data


def test_pydantic_django_model_serializer_successful_initialization(
    pydantic_django_model_test_serializer,
):
    pydantic_django_model_test_serializer()

    assert True  # exception was not thrown


def test_pydantic_django_model_serializer_invalid_fields_subset_error(
    pydantic_django_model_test_serializer,
):
    class InvalidPydanticModel(BaseModel):
        class Config:
            orm_mode = True

        field_a: str
        field_c: int

    pydantic_django_model_test_serializer.pydantic_model = InvalidPydanticModel

    with pytest.raises(
        ImproperlyConfigured, match='Pydantic model fields is not subset of django model fields'
    ):
        pydantic_django_model_test_serializer()


def test_pydantic_django_model_serializer_meta_fields_error(pydantic_django_model_test_serializer):
    pydantic_django_model_test_serializer.Meta.fields = ['field_a']

    with pytest.raises(
        ImproperlyConfigured,
        match='Meta.fields does not affect this serializer behavior. Remove this attribute',
    ):
        pydantic_django_model_test_serializer()


def test_pydantic_django_model_serializer_orm_mode_disabled_error(
    pydantic_django_model_test_serializer,
):
    class InvalidPydanticModel(BaseModel):
        field_a: str

    pydantic_django_model_test_serializer.pydantic_model = InvalidPydanticModel

    with pytest.raises(
        ImproperlyConfigured,
        match='pydantic_model.Config.orm_mode must be True for this serializer',
    ):
        pydantic_django_model_test_serializer()


def test_pydantic_django_model_serializer_to_representation_success(
    pydantic_django_model_test_serializer,
    django_test_model,
    pydantic_test_model_data,
    serialized_pydantic_test_model_data,
):
    model_instance = django_test_model(**pydantic_test_model_data)

    representation = pydantic_django_model_test_serializer().to_representation(model_instance)

    assert representation == serialized_pydantic_test_model_data


def test_pydantic_django_model_serializer_create_success(
    pydantic_django_model_test_serializer,
    django_test_model,
    pydantic_test_model_data,
    serialized_pydantic_test_model_data,
):
    serializer = pydantic_django_model_test_serializer(data=pydantic_test_model_data)

    serializer.is_valid()
    serializer.create(serializer.data)

    django_test_model._meta.default_manager.create.assert_called_once_with(
        **serialized_pydantic_test_model_data
    )


def test_pydantic_django_model_serializer_update_success(
    pydantic_django_model_test_serializer,
    django_test_model,
    pydantic_test_model_data,
    serialized_pydantic_test_model_data,
):
    serializer = pydantic_django_model_test_serializer(data=pydantic_test_model_data)
    model_instance = django_test_model(**pydantic_test_model_data)
    model_instance.field_b = 100500

    serializer.is_valid()
    serializer.update(model_instance, serializer.data)

    assert model_instance.field_b == serialized_pydantic_test_model_data['field_b']
    model_instance.save.assert_called_once()



from __future__ import annotations

import pytest

from restdoctor.utils.api_format import get_available_format, get_filter_formats


@pytest.mark.parametrize(
    'formats,api_format,expected',
    (
        (('full', 'compact', 'big'), 'compact', ['compact']),
        (('full', 'compact'), 'full', ['full']),
        (('full', 'compact:{3,2,1}'), 'compact:2', ['compact:1', 'compact:2']),
        (('full', 'compact:{3,2,1}'), 'full', ['full']),
        (('full', 'test:32'), 'full', ['full']),
        (('full', 'test:{32}'), 'test:32', ['test:32']),
        (('full', 'my_name:1', 'mobile:1:2'), 'my_name:1', ['my_name:1']),
        (('full', 'mobile:1:2'), 'mobile:1:2', ['mobile:1:2']),
        (('full', 'my_name:{32}'), 'my_name:32', ['my_name:32']),
        (
            ('full', 'my_name:{1,2}', 'my_name:{15,12,30}'),
            'my_name:15',
            ['my_name:12', 'my_name:15'],
        ),
        (('full', 'my_name:{32,433,14}'), 'my_name:32', ['my_name:14', 'my_name:32']),
    ),
)
def test_get_filter_formats(formats, api_format, expected):
    api_formats = get_filter_formats(formats, api_format)

    assert api_formats == expected


@pytest.mark.parametrize(
    'formats,expected',
    (
        (('full', 'compact'), ['full', 'compact']),
        (('full', 'compact:{3,2,1}'), ['full', 'compact:1', 'compact:2', 'compact:3']),
        (('full', 'test:32'), ['full', 'test:32']),
        (('full', 'test:{32}'), ['full', 'test:32']),
        (('full', 'my_name:1'), ['full', 'my_name:1']),
        (('full', 'mobile:1:2'), ['full', 'mobile:1:2']),
        (('full', 'mobile:'), ['full', 'mobile:']),
        (('full', 'my_name:3!2'), ['full', 'my_name:3!2']),
        (('test', 'my_name:{32,433,14}'), ['test', 'my_name:14', 'my_name:32', 'my_name:433']),
    ),
)
def test_get_available_format(formats, expected):
    api_formats = get_available_format(formats)

    assert api_formats == expected



import pytest
from rest_framework.exceptions import ErrorDetail

from restdoctor.rest_framework.exception_handlers import _get_full_details


@pytest.mark.parametrize(
    ('details', 'expected_result'),
    [
        (ErrorDetail('Empty field', code='blank'), {'message': 'Empty field', 'code': 'blank'}),
        (
            [
                ErrorDetail('Empty field', code='blank'),
                ErrorDetail('Invalid phone', code='invalid'),
            ],
            [
                {'message': 'Empty field', 'code': 'blank'},
                {'message': 'Invalid phone', 'code': 'invalid'},
            ],
        ),
        (
            {
                'phone': [
                    ErrorDetail('Empty field', code='blank'),
                    ErrorDetail('Invalid phone', code='invalid'),
                ],
            },
            [
                {'field': 'phone', 'message': 'Empty field', 'code': 'blank'},
                {'field': 'phone', 'message': 'Invalid phone', 'code': 'invalid'},
            ],
        ),
        (
            {
                'phones.1.phone': [
                    ErrorDetail(string='Empty field', code='blank'),
                    ErrorDetail(string='Invalid phone', code='invalid'),
                ],
                'phones.2.comment': [
                    ErrorDetail(string='Empty field', code='blank'),
                    ErrorDetail(string='Invalid phone', code='invalid'),
                ],
                'phones.3.phone': [
                    ErrorDetail(string='Empty field', code='blank'),
                    ErrorDetail(string='Invalid phone', code='invalid'),
                ],
            },
            [
                {'message': 'Empty field', 'code': 'blank', 'field': 'phones.1.phone'},
                {'message': 'Invalid phone', 'code': 'invalid', 'field': 'phones.1.phone'},
                {'message': 'Empty field', 'code': 'blank', 'field': 'phones.2.comment'},
                {'message': 'Invalid phone', 'code': 'invalid', 'field': 'phones.2.comment'},
                {'message': 'Empty field', 'code': 'blank', 'field': 'phones.3.phone'},
                {'message': 'Invalid phone', 'code': 'invalid', 'field': 'phones.3.phone'},
            ],
        ),
        (
            {
                'phones': [
                    {
                        'phones.1.phone': [
                            ErrorDetail(string='Empty field', code='blank'),
                            ErrorDetail(string='Invalid phone', code='invalid'),
                        ],
                        'phones.2.comment': [
                            ErrorDetail(string='Empty field', code='blank'),
                            ErrorDetail(string='Invalid phone', code='invalid'),
                        ],
                        'phones.3.phone': [
                            ErrorDetail(string='Empty field', code='blank'),
                            ErrorDetail(string='Invalid phone', code='invalid'),
                        ],
                    },
                ],
            },
            [
                {'message': 'Empty field', 'code': 'blank', 'field': 'phones.1.phone'},
                {'message': 'Invalid phone', 'code': 'invalid', 'field': 'phones.1.phone'},
                {'message': 'Empty field', 'code': 'blank', 'field': 'phones.2.comment'},
                {'message': 'Invalid phone', 'code': 'invalid', 'field': 'phones.2.comment'},
                {'message': 'Empty field', 'code': 'blank', 'field': 'phones.3.phone'},
                {'message': 'Invalid phone', 'code': 'invalid', 'field': 'phones.3.phone'},
            ],
        ),
    ],
    ids=[
        'Single error',
        'List of errors',
        'Errors in single field',
        'Errors in multiple fields',
        'Nested errors in multiple fields',
    ],
)
def test__get_full_details(details, expected_result):
    errors_data = _get_full_details(details)

    assert errors_data == expected_result



from __future__ import annotations

from rest_framework.serializers import BaseSerializer


class SerializerA(BaseSerializer):
    pass


class SerializerB(BaseSerializer):
    pass


class SerializerC(BaseSerializer):
    pass


class SerializerD(BaseSerializer):
    pass


class SerializerE(BaseSerializer):
    pass


class SerializerF(BaseSerializer):
    pass


class SerializerG(BaseSerializer):
    pass


serializer_class_map_with_default = {
    'default': SerializerA,
    'retrieve': {'request': SerializerB},
    'list': {'request': SerializerB, 'response': SerializerB},
    'create': {'request': SerializerC, 'response': SerializerB},
}

serializer_class_map_no_default = {
    'retrieve': {'request': SerializerB},
    'list': {'request': SerializerB, 'response': SerializerB},
    'create': {'request': SerializerC, 'response': SerializerB},
}


serializer_class_map_with_format = {
    'default': SerializerA,
    'default.compact': SerializerB,
    'default.version:2': SerializerG,
    'retrieve': {'response': SerializerB, 'response.compact': SerializerC},
    'list': {
        'response.full': SerializerE,
        'response.version:12': SerializerF,
        'response.test:name': SerializerG,
    },
}



from __future__ import annotations

import pytest
from django.core.exceptions import ImproperlyConfigured
from rest_framework.fields import Field
from rest_framework.routers import SimpleRouter

from restdoctor.rest_framework.schema import RestDoctorSchema
from restdoctor.rest_framework.schema.generators import RefsSchemaGenerator30
from tests.stubs.serializers import MyModelWithoutHelpTextsSerializer, WithMethodFieldSerializer
from tests.test_unit.test_schema.conftest import UrlConf
from tests.test_unit.test_schema.stubs import DefaultViewSet, DefaultViewSetWithOperationId


@pytest.mark.parametrize(
    ('field', 'expected_help_text'), [(Field(), None), (Field(help_text='help_text'), 'help_text')]
)
def test_get_field_description_not_raises_success_case(field, expected_help_text):
    schema = RestDoctorSchema()

    result = schema.get_field_description(field)

    assert result == expected_help_text


def test_get_field_description_strict_fail_case(settings):
    settings.API_STRICT_SCHEMA_VALIDATION = True
    schema = RestDoctorSchema()

    with pytest.raises(ImproperlyConfigured):
        schema.get_field_description(Field())


def test_get_field_description_success_case(settings):
    settings.API_STRICT_SCHEMA_VALIDATION = True
    schema = RestDoctorSchema()

    result = schema.get_field_description(Field(help_text='help_text'))

    assert result == 'help_text'


def test_get_field_description_model_field_not_raises_success_case():
    schema = RestDoctorSchema()

    result = schema.get_field_description(
        MyModelWithoutHelpTextsSerializer().fields['abstract_field']
    )

    assert result is None


def test_get_field_description_model_field_strict_fail_case(settings):
    settings.API_STRICT_SCHEMA_VALIDATION = True
    schema = RestDoctorSchema()

    with pytest.raises(ImproperlyConfigured):
        schema.get_field_description(MyModelWithoutHelpTextsSerializer().fields['abstract_field'])


def test_get_field_description_model_field_success_case(settings):
    settings.API_STRICT_SCHEMA_VALIDATION = True
    schema = RestDoctorSchema()

    result = schema.get_field_description(MyModelWithoutHelpTextsSerializer().fields['timestamp'])

    assert result == 'Event timestamp'


@pytest.mark.parametrize(
    'field',
    [
        'field',
        'optional_field',
        'many_field',
        'list_field',
        'optional_many_field',
        'optional_list_field',
        'multiple_choice_field',
        'type_checking_field',
    ],
)
def test_check_annotations_success_case(settings, field):
    settings.API_STRICT_SCHEMA_VALIDATION = True
    schema = RestDoctorSchema()
    serializer = WithMethodFieldSerializer()

    schema.map_field(serializer.fields[field])


@pytest.mark.parametrize(
    'field',
    [
        'incorrect_field',
        'incorrect_optional_field',
        'incorrect_many_field',
        'incorrect_list_field',
        'incorrect_optional_many_field',
        'incorrect_multiple_choice_field',
    ],
)
def test_check_annotations_fail_case(settings, field):
    settings.API_STRICT_SCHEMA_VALIDATION = True
    schema = RestDoctorSchema()
    serializer = WithMethodFieldSerializer()

    with pytest.raises(ImproperlyConfigured):
        schema.map_field(serializer.fields[field])


def test_duplicate_operation_ids_fail_case(make_urlconf):
    urlconf = make_urlconf(('foo', DefaultViewSet, 'foo'), ('bar', DefaultViewSet, 'bar'))
    generator = RefsSchemaGenerator30(urlconf=urlconf)

    with pytest.raises(Exception, match='duplicates existing id'):
        generator.get_schema()


def test_duplicate_operation_ids_success_case(make_urlconf):
    urlconf = make_urlconf(
        ('foo', DefaultViewSet, 'foo'), ('bar', DefaultViewSetWithOperationId, 'bar')
    )
    generator = RefsSchemaGenerator30(urlconf=urlconf)

    schema = generator.get_schema()
    pass



from __future__ import annotations

import functools
import typing

from restdoctor.constants import DEFAULT_PREFIX_FORMAT_VERSION


def _find_format_range(name_format: str) -> typing.List[int]:
    is_block = False
    current_number = []
    versions_pool = []
    for word in name_format:
        if word == '{':
            is_block = True
        if is_block and word.isdigit():
            current_number.append(word)
        if is_block and word == ',':
            versions_pool.append(int(''.join(current_number)))
            current_number = []
        if word == '}':
            if current_number:
                versions_pool.append(int(''.join(current_number)))
            break
    versions_pool = sorted(versions_pool)
    return versions_pool


@functools.lru_cache
def generate_format(api_format: str) -> typing.List[str]:
    if DEFAULT_PREFIX_FORMAT_VERSION not in api_format:
        return [api_format]

    api_format_name, api_format_prefix = api_format.split(DEFAULT_PREFIX_FORMAT_VERSION, 1)
    format_range = _find_format_range(api_format_prefix)
    if not format_range:
        return [api_format]
    result = []
    for version in format_range:
        result.append(f'{api_format_name}{DEFAULT_PREFIX_FORMAT_VERSION}{version}')
    return result


def get_available_format(available_formats: typing.Tuple[str, ...]) -> typing.List[str]:
    result = []
    for api_format in available_formats:
        result.extend(generate_format(api_format))
    return result


def get_filter_formats(
    available_formats: typing.Tuple[str, ...], requested_format: str
) -> typing.List[str]:

    for api_format in available_formats:
        result = []
        for format_name in generate_format(api_format):
            result.append(format_name)
            if format_name == requested_format:
                return result
    return [requested_format]



from __future__ import annotations

import decimal
import re

import pytest
from django.core.validators import MaxValueValidator, MinValueValidator, URLValidator
from rest_framework import serializers
from rest_framework.fields import (
    BooleanField,
    CharField,
    DateField,
    DateTimeField,
    DecimalField,
    DurationField,
    EmailField,
    Field,
    FileField,
    FloatField,
    ImageField,
    IntegerField,
    IPAddressField,
    JSONField,
    ReadOnlyField,
    RegexField,
    SlugField,
    TimeField,
    URLField,
    UUIDField,
)
from rest_framework_gis.fields import GeometryField

from restdoctor.rest_framework.schema import RestDoctorSchema

url_pattern = URLValidator.regex.pattern.replace('\\Z', '\\z')


@pytest.mark.parametrize(
    ('field', 'expected_schema'),
    [
        (Field(), {'type': 'string'}),
        (ReadOnlyField(), {'readOnly': True, 'type': 'string'}),
        (Field(write_only=True), {'writeOnly': True, 'type': 'string'}),
        (Field(allow_null=True), {'nullable': True, 'type': 'string'}),
        (Field(default='default'), {'default': 'default', 'type': 'string'}),
        (Field(help_text='help_text'), {'description': 'help_text', 'type': 'string'}),
    ],
)
def test_basic_fields_schema(field, expected_schema):
    schema = RestDoctorSchema()

    result = schema._get_field_schema(field)

    assert result == expected_schema


@pytest.mark.parametrize(
    ('field', 'expected_schema'),
    [
        (
            # AutoField, BigIntegerField, IntegerField, PositiveIntegerField,
            # PositiveSmallIntegerField, SmallIntegerField
            IntegerField(validators=[MinValueValidator(10)]),
            {'type': 'integer', 'minimum': 10},
        ),
        (
            # BigIntegerField
            IntegerField(max_value=2 ** 32),
            {'type': 'integer', 'maximum': 2 ** 32, 'format': 'int64'},
        ),
        (
            # PositiveIntegerField, PositiveSmallIntegerField
            IntegerField(min_value=0),
            {'type': 'integer', 'minimum': 0},
        ),
        (
            # BooleanField, NullBooleanField
            BooleanField(),
            {'type': 'boolean'},
        ),
        (
            # NullBooleanField
            BooleanField(allow_null=True),
            {'type': 'boolean', 'nullable': True},
        ),
        (
            # CharField, TextField
            CharField(allow_blank=True, max_length=10, min_length=2),
            {'type': 'string', 'maxLength': 10, 'minLength': 2},
        ),
        (
            # CharField, TextField
            CharField(allow_blank=False),
            {'type': 'string'},
        ),
        (
            # CharField, TextField
            CharField(trim_whitespace=False),
            {'type': 'string'},
        ),
        (
            # DateField
            DateField(),
            {'type': 'string', 'format': 'date'},
        ),
        (
            # DateTimeField
            DateTimeField(),
            {'type': 'string', 'format': 'date-time'},
        ),
        (
            # DecimalField
            DecimalField(
                max_digits=4,
                decimal_places=2,
                validators=[MinValueValidator(10), MaxValueValidator(20)],
            ),
            {
                'type': 'string',
                'format': 'decimal',
                'multipleOf': 0.01,
                'maximum': 20,
                'minimum': 10,
            },
        ),
        (
            # DecimalField
            DecimalField(max_digits=5, decimal_places=2, validators=[MinValueValidator(0)]),
            {
                'type': 'string',
                'format': 'decimal',
                'multipleOf': 0.01,
                'maximum': 1000,
                'minimum': 0,
            },
        ),
        (
            # DecimalField decimal
            DecimalField(
                max_digits=5,
                decimal_places=2,
                validators=[
                    MinValueValidator(decimal.Decimal('0.1')),
                    MaxValueValidator(decimal.Decimal('2.1222')),
                ],
            ),
            {
                'type': 'string',
                'format': 'decimal',
                'multipleOf': 0.01,
                'maximum': 2.1222,
                'minimum': 0.1,
            },
        ),
        (
            # DurationField
            DurationField(max_value=300, min_value=100),
            {'type': 'string', 'maximum': 300, 'minimum': 100},
        ),
        (
            # EmailField
            EmailField(),
            {'type': 'string', 'format': 'email'},
        ),
        (
            # JSONField
            JSONField(),
            {'type': 'object'},
        ),
        (
            # FileField
            FileField(),
            {'type': 'string', 'format': 'binary'},
        ),
        (
            # FloatField
            FloatField(),
            {'type': 'number'},
        ),
        (
            # ImageField
            ImageField(),
            {'type': 'string', 'format': 'binary'},
        ),
        (
            # SlugField
            SlugField(),
            {'type': 'string', 'pattern': '^[-a-zA-Z0-9_]+$'},
        ),
        (
            # TimeField
            TimeField(),
            {'type': 'string'},
        ),
        (
            # URLField
            URLField(),
            {'type': 'string', 'format': 'uri', 'pattern': url_pattern},
        ),
        (
            # UUIDField
            UUIDField(),
            {'type': 'string', 'format': 'uuid'},
        ),
        (
            # IPAddressField
            IPAddressField(protocol='IPv6'),
            {'type': 'string', 'format': 'ipv6'},
        ),
        (
            RegexField(re.compile(r'^[-a-zA-Z0-9_]+$')),
            {'type': 'string', 'pattern': '^[-a-zA-Z0-9_]+$'},
        ),
        (
            # IPAddressField
            IPAddressField(protocol='IPv4'),
            {'type': 'string', 'format': 'ipv4'},
        ),
        (
            IntegerField(min_value=100, max_value=200),
            {'type': 'integer', 'minimum': 100, 'maximum': 200},
        ),
        (
            IntegerField(max_value=2147483648),
            {'type': 'integer', 'maximum': 2147483648, 'format': 'int64'},
        ),
        (
            CharField(min_length=5, max_length=10),
            {'type': 'string', 'minLength': 5, 'maxLength': 10},
        ),
        (
            GeometryField(),
            {
                'type': 'object',
                'required': ['type', 'coordinates'],
                'properties': {
                    'type': {'type': 'string', 'enum': ['Point']},
                    'coordinates': {
                        'type': 'array',
                        'items': {'type': 'number', 'format': 'float'},
                        'example': [12.9721, 77.5933],
                        'minItems': 2,
                        'maxItems': 3,
                    },
                },
            },
        ),
    ],
)
def test_fields_validators_schema(field, expected_schema):
    schema = RestDoctorSchema()

    result = schema._get_field_schema(field)

    assert result == expected_schema



from __future__ import annotations

import inspect
import typing

if typing.TYPE_CHECKING:
    from restdoctor.rest_framework.custom_types import ModelObject, GenericRepresentation

from django.core.exceptions import ImproperlyConfigured
from django.db.models import Model as DjangoModel
from pydantic import BaseModel
from pydantic import ValidationError as PydanticValidationError
from rest_framework.exceptions import ValidationError
from rest_framework.fields import empty
from rest_framework.serializers import BaseSerializer as BaseDRFSerializer
from rest_framework.serializers import ModelSerializer as BaseModelSerializer
from rest_framework.serializers import Serializer as BaseSerializer
from rest_framework.serializers import SerializerMetaclass as BaseSerializerMetaclass
from rest_framework.utils import model_meta

from restdoctor.utils.pydantic import convert_pydantic_errors_to_drf_errors


def _is_serializer_subclass(obj: typing.Any) -> bool:
    try:
        return issubclass(obj, Serializer)
    except TypeError:
        return False


def _get_serializer_fields(serializer_cls: Serializer) -> list[str]:
    try:
        return list(serializer_cls.Meta.fields)
    except AttributeError:
        return list(serializer_cls._declared_fields)


class _DeferredMetaFields(tuple):
    pass


def extend_meta_fields(*args: Serializer | str | None) -> _DeferredMetaFields:
    return _DeferredMetaFields(args)


class SerializerMetaclass(BaseSerializerMetaclass):
    def __new__(cls, name: str, bases: tuple, attrs: dict[str, typing.Any]) -> SerializerMetaclass:
        meta = attrs.get('Meta')
        if meta:
            fields = getattr(meta, 'fields', None)
            if isinstance(fields, _DeferredMetaFields):
                meta.fields = cls._extend_meta_fields(fields, bases)
        return super().__new__(cls, name, bases, attrs)

    @classmethod
    def _extend_meta_fields(cls, deferred_fields: _DeferredMetaFields, bases: tuple) -> list[str]:
        fields: list[str] = []
        serializer_in_fields = False
        for arg in deferred_fields:
            if isinstance(arg, str):
                fields.append(arg)
            elif _is_serializer_subclass(arg):
                fields.extend(_get_serializer_fields(arg))
                serializer_in_fields = True
            else:
                raise TypeError(f'str or Serializer subclass expected, got {arg!r}')
        if serializer_in_fields:
            return fields
        parent_fields: list[str] = []
        for parent_cls in bases:
            parent_fields.extend(_get_serializer_fields(parent_cls))
        return parent_fields + fields


class ModelSerializerMetaclass(SerializerMetaclass):
    def __new__(
        cls, name: str, bases: tuple, attrs: dict[str, typing.Any]
    ) -> ModelSerializerMetaclass:
        serializer_cls = super().__new__(cls, name, bases, attrs)
        meta = attrs.get('Meta')
        if meta:
            if not hasattr(meta, 'model'):
                model = cls._get_meta_model(serializer_cls)
                if model:
                    meta.model = model
            fields = getattr(meta, 'fields', None)
            if isinstance(fields, _DeferredMetaFields):
                meta.fields = cls._extend_meta_fields(fields, bases)
        return serializer_cls

    @classmethod
    def _get_meta_model(cls, serializer_cls: ModelSerializer) -> typing.Optional[ModelObject]:
        for parent_cls in inspect.getmro(serializer_cls):
            parent_meta = parent_cls.__dict__.get('Meta')
            if parent_meta and hasattr(parent_meta, 'model'):
                return parent_meta.model
        return None


class Serializer(BaseSerializer, metaclass=SerializerMetaclass):
    pass


class EmptySerializer(Serializer):
    pass


class ModelSerializer(BaseModelSerializer, Serializer, metaclass=ModelSerializerMetaclass):
    pass


class PydanticSerializer(BaseDRFSerializer):
    """Serializer for pydantic models."""

    class Meta:
        model: typing.Optional[typing.Type[DjangoModel]] = None

    pydantic_model: typing.Type[BaseModel]

    def __new__(cls, *args: list[typing.Any], **kwargs: dict[str, typing.Any]):  # type: ignore
        if not hasattr(cls, 'pydantic_model'):
            raise AttributeError(
                'Class attribute "pydantic_model" is mandatory for this serializer'
            )
        if not inspect.isclass(cls.pydantic_model) or not issubclass(cls.pydantic_model, BaseModel):
            raise AttributeError(
                'Class attribute "pydantic_model" must be an instance of pydantic.BaseModel'
            )
        cls._validate_django_model()
        return super().__new__(cls, *args, **kwargs)

    def __init__(
        self,
        instance: ModelObject | None = None,
        data: dict[str, typing.Any] | empty = empty,
        **kwargs: dict[str, typing.Any],
    ):
        self._pydantic_instance: BaseModel | None = None
        self._setup_create_update_methods()
        super().__init__(instance=instance, data=data, **kwargs)

    @classmethod
    def _validate_django_model(cls) -> None:
        model = getattr(cls.Meta, 'model', None)
        if not model:
            return
        cls._validate_django_model_fields(model)
        cls._validate_meta_fields_not_exist()
        cls._validate_orm_mode_enabled()

    @classmethod
    def _validate_django_model_fields(cls, model: DjangoModel) -> None:
        pydantic_fields_set = set(cls.pydantic_model.__fields__.keys())
        model_info = model_meta.get_field_info(model)
        model_fields = list(model_info.fields.keys())
        model_fields.extend(model_info.relations.keys())
        if not pydantic_fields_set.issubset(model_fields):
            raise ImproperlyConfigured('Pydantic model fields is not subset of django model fields')

    @classmethod
    def _validate_meta_fields_not_exist(cls) -> None:
        meta_fields = getattr(cls.Meta, 'fields', None)
        if meta_fields is not None:
            raise ImproperlyConfigured(
                'Meta.fields does not affect this serializer behavior. Remove this attribute'
            )

    @classmethod
    def _validate_orm_mode_enabled(cls) -> None:
        if not getattr(cls.pydantic_model.Config, 'orm_mode', False):
            raise ImproperlyConfigured(
                'pydantic_model.Config.orm_mode must be True for this serializer'
            )

    def to_internal_value(self, data: dict[str, typing.Any]) -> BaseModel:
        try:
            pydantic_model = self.pydantic_model(**data)
        except PydanticValidationError as exc:
            errors = convert_pydantic_errors_to_drf_errors(exc.errors())
            raise ValidationError(errors)
        return pydantic_model

    def to_representation(  # noqa: CAC001
        self, instance: PydanticSerializer | BaseModel | DjangoModel | dict
    ) -> GenericRepresentation:
        # Типы аргумента instance были выяснены экспериментальным путем
        # в ходе тестирования
        if isinstance(instance, dict):
            value = self.to_internal_value(instance).dict()
        elif isinstance(instance, BaseModel):
            value = instance.dict()
        elif isinstance(instance, PydanticSerializer):
            instance.is_valid(raise_exception=True)
            value = instance._pydantic_instance.dict()  # type: ignore
        elif isinstance(instance, DjangoModel):
            value = self._django_model_to_representation(instance)
        else:
            raise TypeError('Unknown type of instance for representation')
        return value

    def is_valid(self, raise_exception: bool = False) -> bool:
        if not hasattr(self, 'initial_data'):
            raise AssertionError(
                'Cannot call `.is_valid()` as no `data=` keyword argument was '
                'passed when instantiating the serializer instance.'
            )
        if not hasattr(self, '_validated_data'):
            try:
                pydantic_instance = self.pydantic_model(**self.initial_data)
                self._pydantic_instance = pydantic_instance
                self._validated_data = pydantic_instance.dict()
            except PydanticValidationError as exc:
                self._validated_data = {}
                self._errors = convert_pydantic_errors_to_drf_errors(exc.errors())
            else:
                self._errors = {}

        if self._errors and raise_exception:
            raise ValidationError(self.errors)

        return not bool(self._errors)

    def _django_model_to_representation(self, instance: DjangoModel) -> GenericRepresentation:
        try:
            value = self.pydantic_model.from_orm(instance).dict()
        except PydanticValidationError as exc:
            raise ValidationError(convert_pydantic_errors_to_drf_errors(exc.errors()))
        return value

    def _setup_create_update_methods(self) -> None:
        self._writable_fields = ()  # for disable unwanted validation in create and update methods
        self.create = BaseModelSerializer.create.__get__(self, self.__class__)
        self.update = BaseModelSerializer.update.__get__(self, self.__class__)



from __future__ import annotations

import datetime

import pytest
from django.db import models
from pydantic import BaseModel, Field, StrictInt, StrictStr

from restdoctor.rest_framework.serializers import PydanticSerializer


class PydanticTestModel(BaseModel):
    class Config:
        orm_mode = True

    created_at: datetime.datetime = Field(default_factory=datetime.datetime.utcnow)
    field_a: StrictStr
    field_b: StrictInt


class DjangoTestModel(models.Model):
    class Meta:
        app_label = 'fake_app'

    field_a = models.TextField()
    field_b = models.TextField()
    created_at = models.DateTimeField()


class TestPydanticSerializer(PydanticSerializer):
    pydantic_model = PydanticTestModel


@pytest.fixture()
def pydantic_test_model() -> PydanticTestModel:
    return PydanticTestModel


@pytest.fixture()
def pydantic_model_test_serializer() -> TestPydanticSerializer:
    return TestPydanticSerializer


@pytest.fixture()
def django_test_model(mocker) -> DjangoTestModel:
    mocker.patch.object(DjangoTestModel._meta, 'default_manager')
    mocker.patch.object(DjangoTestModel, 'save')
    return DjangoTestModel


@pytest.fixture()
def pydantic_django_model_test_serializer(
    pydantic_test_model: BaseModel, django_test_model: models.Model
) -> PydanticSerializer:
    class TestPydanticDjangoModelSerializer(PydanticSerializer):
        class Meta:
            model = django_test_model

        pydantic_model = pydantic_test_model

    return TestPydanticDjangoModelSerializer


@pytest.fixture()
def pydantic_test_model_data() -> dict[str, str | int]:
    return {
        'created_at': datetime.datetime.utcnow().timestamp(),
        'field_a': 'Test text',
        'field_b': 1,
    }


@pytest.fixture()
def serialized_pydantic_test_model_data(
    pydantic_test_model_data, pydantic_test_model
) -> dict[str, str | datetime.datetime]:
    return pydantic_test_model(**pydantic_test_model_data).dict()
